@@ -48,9 +48,9 @@ G_BEGIN_DECLS
 #define LIBBRASERO_BURN_VERSION_MINOR						\
 	27
 #define LIBBRASERO_BURN_VERSION_MICRO						\
-	3
+	4
 #define LIBBRASERO_BURN_AGE							\
-	111
+	202
 
 
 /**
@@ -58,7 +58,8 @@ G_BEGIN_DECLS
  */
 
 gboolean
-brasero_burn_library_start (void);
+brasero_burn_library_start (int *argc,
+                            char **argv []);
 
 void
 brasero_burn_library_stop (void);
