@@ -49,6 +49,7 @@
 
 #include "brasero-tags.h"
 #include "brasero-track-image.h"
+#include "brasero-track-data-cfg.h"
 #include "brasero-session-cfg.h"
 #include "brasero-burn-lib.h"
 #include "brasero-session-helper.h"
@@ -61,13 +62,13 @@ struct _BraseroSessionCfgPrivate
 
 	gchar *output;
 
-	glong caps_sig;
-
 	BraseroSessionError is_valid;
 
 	guint CD_TEXT_modified:1;
 	guint configuring:1;
 	guint disabled:1;
+
+	guint inhibit_flag_sig:1;
 };
 
 #define BRASERO_SESSION_CFG_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_SESSION_CFG, BraseroSessionCfgPrivate))
@@ -75,6 +76,7 @@ struct _BraseroSessionCfgPrivate
 enum
 {
 	IS_VALID_SIGNAL,
+	WRONG_EXTENSION_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -86,6 +88,178 @@ G_DEFINE_TYPE (BraseroSessionCfg, brasero_session_cfg, BRASERO_TYPE_SESSION_SPAN
 #define BRASERO_DEST_SAVED_FLAGS		(BRASERO_DRIVE_PROPERTIES_FLAGS|BRASERO_BURN_FLAG_MULTI)
 #define BRASERO_DRIVE_PROPERTIES_KEY		"/apps/brasero/drives"
 
+/**
+ * Manages the output path
+ */
+
+gboolean
+brasero_session_cfg_has_default_output_path (BraseroSessionCfg *session)
+{
+	BraseroBurnSessionClass *klass;
+	BraseroBurnResult result;
+
+	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+	result = klass->get_output_path (BRASERO_BURN_SESSION (session), NULL, NULL);
+	return (result == BRASERO_BURN_OK);
+}
+
+static gboolean
+brasero_session_cfg_wrong_extension_signal (BraseroSessionCfg *session)
+{
+	GValue instance_and_params [1];
+	GValue return_value;
+
+	instance_and_params [0].g_type = 0;
+	g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (session));
+	g_value_set_instance (instance_and_params, session);
+	
+	return_value.g_type = 0;
+	g_value_init (&return_value, G_TYPE_BOOLEAN);
+	g_value_set_int (&return_value, FALSE);
+
+	g_signal_emitv (instance_and_params,
+			session_cfg_signals [WRONG_EXTENSION_SIGNAL],
+			0,
+			&return_value);
+
+	g_value_unset (instance_and_params);
+
+	return g_value_get_boolean (&return_value);
+}
+
+static BraseroBurnResult
+brasero_session_cfg_set_output_image (BraseroBurnSession *session,
+				      BraseroImageFormat format,
+				      const gchar *image,
+				      const gchar *toc)
+{
+	gchar *dot;
+	BraseroBurnResult result;
+	BraseroBurnSessionClass *klass;
+	const gchar *suffixes [] = {".iso",
+				    ".toc",
+				    ".cue",
+				    ".toc",
+				    NULL };
+
+	/* First set all information */
+	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+	result = klass->set_output_image (session,
+					  format,
+					  image,
+					  toc);
+
+	if (!image && !toc)
+		return result;
+
+	if (format == BRASERO_IMAGE_FORMAT_NONE)
+		format = brasero_burn_session_get_output_format (session);
+
+	if (format == BRASERO_IMAGE_FORMAT_NONE)
+		return result;
+
+	if (format & BRASERO_IMAGE_FORMAT_BIN) {
+		dot = g_utf8_strrchr (image, -1, '.');
+		if (!strcmp (suffixes [0], dot)) {
+			gboolean res;
+
+			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
+			if (res)
+				brasero_image_format_fix_path_extension (format, FALSE, image);
+		}
+	}
+	else {
+		dot = g_utf8_strrchr (toc, -1, '.');
+
+		if (format & BRASERO_IMAGE_FORMAT_CLONE
+		&& !strcmp (suffixes [1], dot)) {
+			gboolean res;
+
+			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
+			if (res)
+				brasero_image_format_fix_path_extension (format, FALSE, toc);
+		}
+		else if (format & BRASERO_IMAGE_FORMAT_CUE
+		     && !strcmp (suffixes [2], dot)) {
+			gboolean res;
+
+			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
+			if (res)
+				brasero_image_format_fix_path_extension (format, FALSE, toc);
+		}
+		else if (format & BRASERO_IMAGE_FORMAT_CDRDAO
+		     && !strcmp (suffixes [3], dot)) {
+			gboolean res;
+
+			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
+			if (res)
+				brasero_image_format_fix_path_extension (format, FALSE, toc);
+		}
+	}
+
+	return result;
+}
+
+static BraseroBurnResult
+brasero_session_cfg_get_output_path (BraseroBurnSession *session,
+				     gchar **image,
+				     gchar **toc)
+{
+	gchar *path = NULL;
+	BraseroBurnResult result;
+	BraseroImageFormat format;
+	BraseroBurnSessionClass *klass;
+
+	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+
+	result = klass->get_output_path (session,
+					 toc,
+					 image);
+	if (result == BRASERO_BURN_OK)
+		return result;
+
+	format = brasero_burn_session_get_output_format (session);
+	path = brasero_image_format_get_default_path (format);
+
+	switch (format) {
+	case BRASERO_IMAGE_FORMAT_BIN:
+		if (image)
+			*image = path;
+		break;
+
+	case BRASERO_IMAGE_FORMAT_CDRDAO:
+	case BRASERO_IMAGE_FORMAT_CLONE:
+	case BRASERO_IMAGE_FORMAT_CUE:
+		if (toc)
+			*toc = path;
+
+		if (image)
+			*image = brasero_image_format_get_complement (format, path);
+		break;
+
+	default:
+		g_free (path);
+		return BRASERO_BURN_ERR;
+	}
+
+	return BRASERO_BURN_OK;
+}
+
+static BraseroImageFormat
+brasero_session_cfg_get_output_format (BraseroBurnSession *session)
+{
+	BraseroBurnSessionClass *klass;
+	BraseroImageFormat format;
+
+	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+	format = klass->get_output_format (session);
+
+	if (format == BRASERO_IMAGE_FORMAT_NONE)
+		format = brasero_burn_session_get_default_output_format (session);
+
+	return format;
+}
+
 /**
  * Get a key to save parameters through GConf
  */
@@ -117,7 +291,7 @@ brasero_session_cfg_get_gconf_key (BraseroSessionCfg *self,
 	else
 		display_name = g_strdup ("File");
 
-	display_name = display_name ? display_name : "";
+	display_name = display_name ? display_name : g_strdup ("");
 
 	disc_type = gconf_escape_key (brasero_medium_get_type_string (medium), -1);
 	if (!disc_type) {
@@ -248,7 +422,7 @@ brasero_session_cfg_save_drive_properties (BraseroSessionCfg *self,
 }
 
 static void
-brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
+brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
 						BraseroBurnFlag flags)
 {
 	BraseroMedia media;
@@ -256,6 +430,7 @@ brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
 	BraseroDrive *drive;
 	BraseroMedium *medium;
 	BraseroBurnResult result;
+	BraseroBurnFlag original_flags;
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
@@ -270,25 +445,27 @@ brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
 
 	media = brasero_medium_get_status (medium);
 
-	/* add flags then wipe out flags from session to check them one by one */
-	flags |= brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
-	brasero_burn_session_remove_flag (BRASERO_BURN_SESSION (self), flags);
+	/* This prevents signals to be emitted while (re-) adding them one by one */
+	priv->inhibit_flag_sig = TRUE;
+
+	original_flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
+	if (original_flags == flags)
+		return;
+
+	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_NONE);
 
 	priv->supported = BRASERO_BURN_FLAG_NONE;
 	priv->compulsory = BRASERO_BURN_FLAG_NONE;
 	result = brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
 						      &priv->supported,
 						      &priv->compulsory);
+
 	if (result != BRASERO_BURN_OK) {
-		brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), flags);
+		brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), original_flags | flags);
+		priv->inhibit_flag_sig = FALSE;
 		return;
 	}
 
-	/* These are always supported and better be set. */
-	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self),
-					BRASERO_BURN_FLAG_CHECK_SIZE|
-					BRASERO_BURN_FLAG_NOGRACE);
-
 	for (flag = BRASERO_BURN_FLAG_EJECT; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
 		/* see if this flag was originally set */
 		if (!(flags & flag))
@@ -343,7 +520,8 @@ brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
 		brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
 					       BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE);
 
-		if (priv->supported & BRASERO_BURN_FLAG_FAST_BLANK)
+		if (priv->supported & BRASERO_BURN_FLAG_FAST_BLANK
+		&& (media & BRASERO_MEDIUM_UNFORMATTED) == 0)
 			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
 						       BRASERO_BURN_FLAG_FAST_BLANK);
 
@@ -355,8 +533,11 @@ brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
 	}
 
 	/* When copying with same drive don't set write mode, it'll be set later */
-	if (!brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))) {
-		/* use DAO whenever it's possible */
+	if (!brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))
+	&&  !(media & BRASERO_MEDIUM_DVD)) {
+		/* use DAO whenever it's possible except for DVDs otherwise
+		 * wodime which claims to support it will be used by default
+		 * instead of say growisofs. */
 		if (priv->supported & BRASERO_BURN_FLAG_DAO) {
 			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_DAO);
 
@@ -371,10 +552,37 @@ brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
 		}
 	}
 
+	/* allow flag changed signal again */
+	priv->inhibit_flag_sig = FALSE;
+
+	/* These are always supported and better be set.
+	 * Set them now to trigger the "flags-changed" signal */
+	brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
+	                               BRASERO_BURN_FLAG_CHECK_SIZE|
+	                               BRASERO_BURN_FLAG_NOGRACE);
+
 	/* Always save flags */
 	brasero_session_cfg_save_drive_flags (self, medium);
 }
 
+static void
+brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
+						BraseroBurnFlag flags)
+{
+	/* add flags then wipe out flags from session to check them one by one */
+	flags |= brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
+	brasero_session_cfg_set_drive_properties_flags (self, flags);
+}
+
+static void
+brasero_session_cfg_rm_drive_properties_flags (BraseroSessionCfg *self,
+					       BraseroBurnFlag flags)
+{
+	/* add flags then wipe out flags from session to check them one by one */
+	flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self)) & (~flags);
+	brasero_session_cfg_set_drive_properties_flags (self, flags);
+}
+
 static void
 brasero_session_cfg_set_drive_properties (BraseroSessionCfg *self)
 {
@@ -392,10 +600,7 @@ brasero_session_cfg_set_drive_properties (BraseroSessionCfg *self)
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
 	/* The next two must work as they were checked earlier */
-	source = brasero_track_type_new ();
-	brasero_burn_session_get_input_type (BRASERO_BURN_SESSION (self), source);
 	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
-
 	medium = brasero_drive_get_medium (drive);
 	if (!medium || brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE)
 		return;
@@ -438,6 +643,8 @@ brasero_session_cfg_set_drive_properties (BraseroSessionCfg *self)
 
 	g_object_unref (client);
 
+	source = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (BRASERO_BURN_SESSION (self), source);
 	if (brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))) {
 		/* Special case */
 		if (value) {
@@ -466,6 +673,7 @@ brasero_session_cfg_set_drive_properties (BraseroSessionCfg *self)
 		flags = gconf_value_get_int (value) & BRASERO_DEST_SAVED_FLAGS;
 		gconf_value_free (value);
 	}
+	brasero_track_type_free (source);
 
 	brasero_session_cfg_add_drive_properties_flags (self, flags);
 }
@@ -498,9 +706,9 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 	BraseroDrive *burner;
 	GValue *value = NULL;
 	/* in sectors */
-	gint64 session_size;
-	gint64 max_sectors;
-	gint64 disc_size;
+	goffset session_size;
+	goffset max_sectors;
+	goffset disc_size;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
@@ -639,9 +847,9 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 			       session_cfg_signals [IS_VALID_SIGNAL],
 			       0);
 		return;
-
 	}
-	else if (result == BRASERO_BURN_ERR) {
+
+	if (result == BRASERO_BURN_ERR) {
 		GError *error;
 
 		error = brasero_status_get_error (status);
@@ -669,12 +877,13 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 	if (brasero_track_type_is_empty (source)) {
 		brasero_track_type_free (source);
 
-		priv->is_valid = BRASERO_SESSION_NOT_SUPPORTED;
+		priv->is_valid = BRASERO_SESSION_EMPTY;
 		g_signal_emit (self,
 			       session_cfg_signals [IS_VALID_SIGNAL],
 			       0);
 		return;
 	}
+	/* it can be empty with an empty track */
 
 	if (brasero_track_type_get_has_medium (source)
 	&&  brasero_track_type_get_medium_type (source) == BRASERO_MEDIUM_NONE) {
@@ -847,7 +1056,81 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 }
 
 static void
-brasero_session_cfg_input_changed (BraseroBurnSession *session)
+brasero_session_cfg_session_loaded (BraseroTrackDataCfg *track,
+				    BraseroMedium *medium,
+				    gboolean is_loaded,
+				    BraseroSessionCfg *session)
+{
+	if (is_loaded) {
+		/* Set the correct medium and add the flag */
+		brasero_burn_session_set_burner (BRASERO_BURN_SESSION (session),
+						 brasero_medium_get_drive (medium));
+
+		brasero_session_cfg_add_drive_properties_flags (session, BRASERO_BURN_FLAG_MERGE);
+	}
+	else
+		brasero_session_cfg_rm_drive_properties_flags (session, BRASERO_BURN_FLAG_MERGE);
+}
+
+static void
+brasero_session_cfg_track_added (BraseroBurnSession *session,
+				 BraseroTrack *track)
+{
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	if (priv->disabled)
+		return;
+
+	if (BRASERO_IS_TRACK_DATA_CFG (track))
+		g_signal_connect (track,
+				  "session-loaded",
+				  G_CALLBACK (brasero_session_cfg_session_loaded),
+				  session);
+
+	/* when that happens it's mostly because a medium source changed, or
+	 * a new image was set. 
+	 * - reload saved flags
+	 * - check if all flags are thereafter supported
+	 * - check available formats for path
+	 * - set one path
+	 */
+	brasero_session_cfg_update (BRASERO_SESSION_CFG (session),
+				    TRUE,
+				    FALSE);
+}
+
+static void
+brasero_session_cfg_track_removed (BraseroBurnSession *session,
+				   BraseroTrack *track,
+				   guint former_position)
+{
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	if (priv->disabled)
+		return;
+
+	/* Just in case */
+	g_signal_handlers_disconnect_by_func (track,
+					      brasero_session_cfg_session_loaded,
+					      session);
+
+	/* when that happens it's mostly because a medium source changed, or
+	 * a new image was set. 
+	 * - reload saved flags
+	 * - check if all flags are thereafter supported
+	 * - check available formats for path
+	 * - set one path
+	 */
+	brasero_session_cfg_update (BRASERO_SESSION_CFG (session),
+				    TRUE,
+				    FALSE);
+}
+
+static void
+brasero_session_cfg_track_changed (BraseroBurnSession *session,
+				   BraseroTrack *track)
 {
 	BraseroSessionCfgPrivate *priv;
 
@@ -909,6 +1192,20 @@ brasero_session_cfg_caps_changed (BraseroPluginManager *manager,
 				    TRUE);
 }
 
+static void
+brasero_session_cfg_flags_changed (BraseroBurnSession *session) 
+{
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+
+	/* when we update the flags we don't want a
+	 * whole series of "flags-changed" emitted.
+	 * so make sure there is just one at the end */
+	if (priv->inhibit_flag_sig)
+		g_signal_stop_emission_by_name (session, "flags-changed");
+}
+
 void
 brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 			       BraseroBurnFlag flags)
@@ -917,7 +1214,6 @@ brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 	BraseroDrive *drive;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
-
 	if ((priv->supported & flags) != flags)
 		return;
 
@@ -996,17 +1292,19 @@ brasero_session_cfg_init (BraseroSessionCfg *object)
 	priv = BRASERO_SESSION_CFG_PRIVATE (object);
 
 	manager = brasero_plugin_manager_get_default ();
-	priv->caps_sig = g_signal_connect (manager,
-					   "caps-changed",
-					   G_CALLBACK (brasero_session_cfg_caps_changed),
-					   object);
+	g_signal_connect (manager,
+	                  "caps-changed",
+	                  G_CALLBACK (brasero_session_cfg_caps_changed),
+	                  object);
 }
 
 static void
 brasero_session_cfg_finalize (GObject *object)
 {
+	BraseroPluginManager *manager;
 	BraseroSessionCfgPrivate *priv;
 	BraseroDrive *drive;
+	GSList *tracks;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (object);
 
@@ -1015,14 +1313,21 @@ brasero_session_cfg_finalize (GObject *object)
 		brasero_session_cfg_save_drive_properties (BRASERO_SESSION_CFG (object),
 							   brasero_drive_get_medium (drive));
 
-	if (priv->caps_sig) {
-		BraseroPluginManager *manager;
+	tracks = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (object));
+	for (; tracks; tracks = tracks->next) {
+		BraseroTrack *track;
 
-		manager = brasero_plugin_manager_get_default ();
-		g_signal_handler_disconnect (manager, priv->caps_sig);
-		priv->caps_sig = 0;
+		track = tracks->data;
+		g_signal_handlers_disconnect_by_func (track,
+						      brasero_session_cfg_session_loaded,
+						      object);
 	}
 
+	manager = brasero_plugin_manager_get_default ();
+	g_signal_handlers_disconnect_by_func (manager,
+	                                      brasero_session_cfg_caps_changed,
+	                                      object);
+
 	G_OBJECT_CLASS (brasero_session_cfg_parent_class)->finalize (object);
 }
 
@@ -1036,10 +1341,27 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 
 	object_class->finalize = brasero_session_cfg_finalize;
 
-	session_class->input_changed = brasero_session_cfg_input_changed;
-	session_class->output_changed = brasero_session_cfg_output_changed;
+	session_class->get_output_path = brasero_session_cfg_get_output_path;
+	session_class->get_output_format = brasero_session_cfg_get_output_format;
+	session_class->set_output_image = brasero_session_cfg_set_output_image;
 
-	session_cfg_signals[IS_VALID_SIGNAL] =
+	session_class->track_added = brasero_session_cfg_track_added;
+	session_class->track_removed = brasero_session_cfg_track_removed;
+	session_class->track_changed = brasero_session_cfg_track_changed;
+	session_class->output_changed = brasero_session_cfg_output_changed;
+	session_class->flags_changed = brasero_session_cfg_flags_changed;
+
+	session_cfg_signals [WRONG_EXTENSION_SIGNAL] =
+		g_signal_new ("wrong_extension",
+		              G_OBJECT_CLASS_TYPE (klass),
+		              G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP | G_SIGNAL_ACTION,
+		              0,
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+			      0,
+		              G_TYPE_NONE);
+	session_cfg_signals [IS_VALID_SIGNAL] =
 		g_signal_new ("is_valid",
 		              G_OBJECT_CLASS_TYPE (klass),
 		              G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP | G_SIGNAL_ACTION,
