@@ -81,13 +81,10 @@ void
 brasero_project_set_source (BraseroProject *project,
 			    BraseroURIContainer *source);
 
-BraseroProjectType
-brasero_project_open_playlist (BraseroProject *project,
-			       const gchar *uri);
-
 BraseroProjectType
 brasero_project_open_project (BraseroProject *project,
-			      const gchar *uri);
+			      const gchar *uri,
+			      gboolean playlist);
 
 gboolean
 brasero_project_save_project (BraseroProject *project);
@@ -107,7 +104,7 @@ brasero_project_register_ui (BraseroProject *project,
 			     GtkUIManager *manager);
 
 void
-brasero_project_set_cover_specifics (BraseroProject *project,
-				     BraseroJacketEdit *cover);
+brasero_project_create_audio_cover (BraseroProject *project,
+				    BraseroJacketEdit *cover);
 
 #endif /* PROJECT_H */
