@@ -1,20 +1,23 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- * brasero
- * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
+ * Brasero
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
- * brasero is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * brasero is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License along
- * with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with brasero.  If not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -84,7 +87,7 @@ brasero_rename_set_show_keep_default (BraseroRename *self,
 			return;
 
 		gtk_combo_box_prepend_text (GTK_COMBO_BOX (priv->combo),
-					     _("<keep current values>"));
+					     _("<Keep current values>"));
 	}
 
 	priv->show_default = show;
@@ -244,7 +247,7 @@ brasero_rename_type_changed (GtkComboBox *combo,
 	BraseroRenamePrivate *priv;
 
 	priv = BRASERO_RENAME_PRIVATE (self);
-	if (!gtk_combo_box_get_active (combo)) {
+	if (gtk_combo_box_get_active (combo) == -1) {
 		gtk_widget_hide (priv->notebook);
 		return;
 	}
@@ -276,7 +279,7 @@ brasero_rename_init (BraseroRename *object)
 	gtk_box_pack_start (GTK_BOX (object), priv->combo, FALSE, FALSE, 0);
 
 	priv->show_default = 1;
-	gtk_combo_box_append_text (GTK_COMBO_BOX (priv->combo), _("<keep current values>"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (priv->combo), _("<Keep current values>"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (priv->combo), _("Insert text"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (priv->combo), _("Delete text"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (priv->combo), _("Substitute text"));
@@ -294,6 +297,7 @@ brasero_rename_init (BraseroRename *object)
 	gtk_widget_show (hbox);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), hbox, NULL);
 
+	/* Translators: This is a verb. This is completed later */
 	label = gtk_label_new (_("Insert"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -305,8 +309,15 @@ brasero_rename_init (BraseroRename *object)
 
 	combo = gtk_combo_box_new_text ();
 	gtk_widget_show (combo);
+
+	/* Translators: This finishes previous action "Insert". It goes like
+	 * this: "Insert" [Entry] "at the beginning". */
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("at the begining"));
+
+	/* Translators: This finishes previous action "Insert". It goes like
+	 * this: "Insert" [Entry] "at the end". */
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("at the end"));
+
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
 	priv->insert_combo = combo;
@@ -330,7 +341,8 @@ brasero_rename_init (BraseroRename *object)
 	gtk_widget_show (hbox);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), hbox, NULL);
 
-	label = gtk_label_new (_("Substitute"));
+	/* Translators: this is a verb */
+	label = gtk_label_new_with_mnemonic (_("_Replace"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
@@ -339,7 +351,13 @@ brasero_rename_init (BraseroRename *object)
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 	priv->substitute_entry = entry;
 
-	label = gtk_label_new (_("by"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+
+	/* Reminder: if this string happens to be used somewhere else in brasero
+	 * we'll need a context with C_() macro */
+	/* Translators: this goes with above verb to say "_Replace" [Entry]
+	 * "with" [Entry]. */
+	label = gtk_label_new (_("with"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
