@@ -32,21 +32,17 @@
 #include <glib-object.h>
 #include <glib/gi18n.h>
 
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkspinbutton.h>
+#include <gtk/gtk.h>
 
 #include <gst/gst.h>
 
 #include "brasero-song-properties.h"
 #include "brasero-time-button.h"
-#include "brasero-utils.h"
-#include "burn-track.h"
+
+#include "brasero-misc.h"
+
+#include "brasero-track.h"
+#include "brasero-track-stream.h"
 
 G_DEFINE_TYPE (BraseroSongProps, brasero_song_props, GTK_TYPE_DIALOG);
 
@@ -62,6 +58,10 @@ struct BraseroSongPropsPrivate {
 
 	GtkWidget *gap;
 	GtkWidget *gap_label;
+
+	guint title_set:1;
+	guint artist_set:1;
+	guint composer_set:1;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -78,7 +78,7 @@ brasero_song_props_update_length (BraseroSongProps *self)
 	start = brasero_time_button_get_value (BRASERO_TIME_BUTTON (self->priv->start));
 	gap = gtk_spin_button_get_value (GTK_SPIN_BUTTON (self->priv->gap)) * GST_SECOND;
 
-	length_str = brasero_utils_get_time_string (BRASERO_AUDIO_TRACK_LENGTH (start, end + gap), TRUE, FALSE);
+	length_str = brasero_units_get_time_string (BRASERO_STREAM_LENGTH (start, end + gap), TRUE, FALSE);
 	gtk_label_set_markup (GTK_LABEL (self->priv->length), length_str);
 	g_free (length_str);
 }
@@ -114,11 +114,32 @@ brasero_song_props_gap_changed_cb (GtkSpinButton *button,
 	start = brasero_time_button_get_value (BRASERO_TIME_BUTTON (self->priv->start));
 	gap = gtk_spin_button_get_value (GTK_SPIN_BUTTON (self->priv->gap)) * GST_SECOND;
 
-	length_str = brasero_utils_get_time_string (BRASERO_AUDIO_TRACK_LENGTH (start, end + gap), TRUE, FALSE);
+	length_str = brasero_units_get_time_string (BRASERO_STREAM_LENGTH (start, end + gap), TRUE, FALSE);
 	gtk_label_set_markup (GTK_LABEL (self->priv->length), length_str);
 	g_free (length_str);
 }
 
+static void
+brasero_song_props_title_set (GtkEntry *entry,
+			      BraseroSongProps *self)
+{
+	self->priv->title_set = TRUE;
+}
+
+static void
+brasero_song_props_artist_set (GtkEntry *entry,
+			       BraseroSongProps *self)
+{
+	self->priv->artist_set = TRUE;
+}
+
+static void
+brasero_song_props_composer_set (GtkEntry *entry,
+				 BraseroSongProps *self)
+{
+	self->priv->composer_set = TRUE;
+}
+
 static void
 brasero_song_props_init (BraseroSongProps *obj)
 {
@@ -153,6 +174,10 @@ brasero_song_props_init (BraseroSongProps *obj)
 	label = gtk_label_new (_("Title:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->title = gtk_entry_new ();
+	g_signal_connect (obj->priv->title,
+			  "changed",
+			  G_CALLBACK (brasero_song_props_title_set),
+			  obj);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->title, 1, 2, 0, 1);
 	gtk_widget_set_tooltip_text (obj->priv->title,
@@ -161,14 +186,22 @@ brasero_song_props_init (BraseroSongProps *obj)
 	label = gtk_label_new (_("Artist:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->artist = gtk_entry_new ();
+	g_signal_connect (obj->priv->artist,
+			  "changed",
+			  G_CALLBACK (brasero_song_props_artist_set),
+			  obj);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->artist, 1, 2, 1, 2);
 	gtk_widget_set_tooltip_text (obj->priv->artist,
 				     _("This information will be written to the disc using CD-TEXT technology. It can be read and displayed by some audio CD players."));
 
-	label = gtk_label_new (_("Composer:\t"));
+	label = gtk_label_new (_("Composer:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->composer = gtk_entry_new ();
+	g_signal_connect (obj->priv->composer,
+			  "changed",
+			  G_CALLBACK (brasero_song_props_composer_set),
+			  obj);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->composer, 1, 2, 2, 3);
 	gtk_widget_set_tooltip_text (obj->priv->composer,
@@ -187,7 +220,7 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
-	title_str = g_strdup_printf ("<b>%s</b>", "Options");
+	title_str = g_strdup_printf ("<b>%s</b>", _("Options"));
 	frame = brasero_utils_pack_properties (title_str,
 					       table,
 					       NULL);
@@ -196,19 +229,19 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj)->vbox), frame, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Song start:\t"));
+	label = gtk_label_new (_("Song start:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->start = brasero_time_button_new ();
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach (GTK_TABLE (table), obj->priv->start, 1, 2, 0, 1, 0, 0, 0, 0);
 
-	label = gtk_label_new (_("Song end:\t"));
+	label = gtk_label_new (_("Song end:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->end = brasero_time_button_new ();
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach (GTK_TABLE (table), obj->priv->end, 1, 2, 1, 2, 0, 0, 0, 0);
 
-	label = gtk_label_new (_("Pause length:\t"));
+	label = gtk_label_new (_("Pause length:"));
 	obj->priv->gap_label = label;
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->gap = gtk_spin_button_new_with_range (0.0, 100.0, 1.0);
@@ -219,7 +252,7 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_widget_set_tooltip_text (obj->priv->gap,
 				     _("Gives the length of the pause that should follow the track"));
 
-	label = gtk_label_new (_("Track length:\t"));
+	label = gtk_label_new (_("Track length:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 	obj->priv->length = gtk_label_new (NULL);
@@ -276,12 +309,26 @@ brasero_song_props_get_properties (BraseroSongProps *self,
 				   gint64 *end,
 				   gint64 *gap)
 {
-	if (artist)
-		*artist = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->artist));
-	if (title)
-		*title = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->title));
-	if (composer)
-		*composer = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->composer));
+	if (artist) {
+		if (self->priv->artist_set)
+			*artist = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->artist));
+		else
+			*artist = NULL;
+	}
+
+	if (title) {
+		if (self->priv->title_set)
+			*title = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->title));
+		else
+			*title = NULL;
+	}
+
+	if (composer) {
+		if (self->priv->composer_set)
+			*composer = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->composer));
+		else
+			*composer = NULL;
+	}
 
 	if (isrc) {
 		const gchar *string;
@@ -369,6 +416,10 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 					   self);
 
 	brasero_song_props_update_length (self);
+
+	self->priv->title_set = FALSE;
+	self->priv->artist_set = FALSE;
+	self->priv->composer_set = FALSE;
 }
 
 static void
