@@ -1,20 +1,23 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- * brasero
- * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
+ * Brasero
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
- * brasero is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * brasero is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License along
- * with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with brasero.  If not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,31 +36,32 @@
 #include "eggtreemultidnd.h"
 
 #include "brasero-tags.h"
+#include "brasero-track-stream-cfg.h"
 
 #include "brasero-misc.h"
-#include "burn-debug.h"
 #include "brasero-app.h"
 #include "brasero-disc.h"
 #include "brasero-io.h"
 #include "brasero-utils.h"
 #include "brasero-video-disc.h"
-#include "brasero-video-project.h"
 #include "brasero-video-tree-model.h"
 #include "brasero-multi-song-props.h"
 #include "brasero-song-properties.h"
 #include "brasero-session-cfg.h"
 #include "brasero-track-stream.h"
+#include "brasero-video-options.h"
 
 typedef struct _BraseroVideoDiscPrivate BraseroVideoDiscPrivate;
 struct _BraseroVideoDiscPrivate
 {
-	GtkWidget *notebook;
 	GtkWidget *tree;
 
 	GtkWidget *message;
 	GtkUIManager *manager;
 	GtkActionGroup *disc_group;
 
+	BraseroIOJobBase *load_dir;
+
 	guint reject_files:1;
 	guint editing:1;
 	guint loading:1;
@@ -185,8 +189,8 @@ brasero_video_disc_name_edited_cb (GtkCellRendererText *cellrenderertext,
 				   BraseroVideoDisc *self)
 {
 	BraseroVideoDiscPrivate *priv;
-	BraseroVideoProject *project;
-	BraseroVideoFile *file;
+	BraseroTrack *track;
+	GtkTreeModel *model;
 	GtkTreePath *path;
 	GtkTreeIter row;
 
@@ -195,109 +199,22 @@ brasero_video_disc_name_edited_cb (GtkCellRendererText *cellrenderertext,
 	priv->editing = 0;
 
 	path = gtk_tree_path_new_from_string (path_string);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
 
 	/* see if this is still a valid path. It can happen a user removes it
 	 * while the name of the row is being edited */
-	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (project), &row, path)) {
+	if (!gtk_tree_model_get_iter (model, &row, path)) {
 		gtk_tree_path_free (path);
 		return;
 	}
 
-	file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (project), path);
+	track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
 
-	brasero_video_project_rename (project, file, text);
-}
+	brasero_track_tag_add_string (track, BRASERO_TRACK_STREAM_TITLE_TAG, text);
 
-static void
-brasero_video_disc_vfs_activity_changed (BraseroVideoProject *project,
-					 gboolean activity,
-					 BraseroVideoDisc *self)
-{
-	GdkCursor *cursor;
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-
-	if (!GTK_WIDGET (self)->window)
-		return;
-
-	if (activity) {
-		cursor = gdk_cursor_new (GDK_WATCH);
-		gdk_window_set_cursor (GTK_WIDGET (self)->window, cursor);
-		gdk_cursor_unref (cursor);
-	}
-	else
-		gdk_window_set_cursor (GTK_WIDGET (self)->window, NULL);
-}
-
-static gboolean
-brasero_video_disc_directory_dialog (BraseroVideoProject *project,
-				     const gchar *uri,
-				     BraseroVideoDisc *self)
-{
-	gint answer;
-	GtkWidget *dialog;
-
-	dialog = brasero_app_dialog (brasero_app_get_default (),
-				     _("Do you want to search for video files inside the directory?"),
-				     GTK_BUTTONS_NONE,
-				     GTK_MESSAGE_WARNING);
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("Directories cannot be added to video discs."));
-
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				_("_Search Directory"), GTK_RESPONSE_OK,
-				NULL);
-
-	gtk_widget_show_all (dialog);
-	answer = gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-
-	if (answer != GTK_RESPONSE_OK)
-		return FALSE;
-
-	return TRUE;
-}
-
-static void
-brasero_video_disc_unreadable_uri_dialog (BraseroVideoProject *project,
-					  GError *error,
-					  const gchar *uri,
-					  BraseroVideoDisc *self)
-{
-	gchar *primary;
-	gchar *name;
-
-	name = g_filename_display_basename (uri);
-	primary = g_strdup_printf (_("\"%s\" could not be opened."), name);
-	brasero_app_alert (brasero_app_get_default (),
-			   primary,
-			   error->message,
-			   GTK_MESSAGE_ERROR);
-	g_free (primary);
-	g_free (name);
-}
-
-static void
-brasero_video_disc_not_video_dialog (BraseroVideoProject *project,
-				     const gchar *uri,
-				     BraseroVideoDisc *self)
-{
-	gchar *primary;
-	gchar *name;
-
-	BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
-	primary = g_strdup_printf (_("\"%s\" does not have a suitable type for video projects."), name);
-	brasero_app_alert (brasero_app_get_default (),
-			   primary,
-			   _("Please only add files with video contents"),
-			   GTK_MESSAGE_ERROR);
-	g_free (primary);
-	g_free (name);
+	/* Advertize change to update view */
+	brasero_track_changed (track);
 }
 
 static BraseroDiscResult
@@ -308,36 +225,36 @@ brasero_video_disc_add_uri_real (BraseroVideoDisc *self,
 				 gint64 end,
 				 GtkTreePath **path_return)
 {
-	BraseroVideoFile *file;
-	BraseroVideoProject *project;
+	GtkTreeModel *model;
+	BraseroSessionCfg*session;
+	BraseroTrack *sibling = NULL;
+	BraseroTrackStreamCfg *track;
 	BraseroVideoDiscPrivate *priv;
-	BraseroVideoFile *sibling = NULL;
 
 	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
 	if (priv->reject_files)
 		return BRASERO_DISC_NOT_READY;
 
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
 
+	/* create track */
+	track = brasero_track_stream_cfg_new ();
+	brasero_track_stream_set_source (BRASERO_TRACK_STREAM (track), uri);
+	brasero_track_stream_set_boundaries (BRASERO_TRACK_STREAM (track), start, end, 0);
+
+	/* insert it in the session */
+	session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (model));
 	if (pos > 0) {
-		GtkTreePath *treepath;
+		GSList *tracks;
 
-		treepath = gtk_tree_path_new ();
-		gtk_tree_path_append_index (treepath, pos);
-		sibling = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (project), treepath);
-		gtk_tree_path_free (treepath);
+		tracks = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (session));
+		sibling = g_slist_nth_data (tracks, pos - 1);
 	}
+	brasero_burn_session_add_track (BRASERO_BURN_SESSION (session), BRASERO_TRACK (track), sibling);
 
-	file = brasero_video_project_add_uri (project,
-					      uri,
-					      NULL,
-					      sibling,
-					      start,
-					      end);
-	if (path_return && file)
-		*path_return = brasero_video_tree_model_file_to_path (BRASERO_VIDEO_TREE_MODEL (project), file);
-
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 1);
+	if (path_return)
+		*path_return = brasero_video_tree_model_track_to_path (BRASERO_VIDEO_TREE_MODEL (model),
+								       BRASERO_TRACK (track));
 
 	return BRASERO_DISC_OK;
 }
@@ -371,11 +288,219 @@ brasero_video_disc_add_uri (BraseroDisc *self,
 	return result;
 }
 
+static void
+brasero_video_disc_add_directory_contents_result (GObject *obj,
+						  GError *error,
+						  const gchar *uri,
+						  GFileInfo *info,
+						  gpointer user_data)
+{
+	gint index;
+
+	/* Check the return status for this file */
+	if (error)
+		return;
+
+	if (g_file_info_get_file_type (info) != G_FILE_TYPE_REGULAR
+	|| !g_file_info_get_attribute_boolean (info, BRASERO_IO_HAS_VIDEO))
+		return;
+
+	index = GPOINTER_TO_INT (user_data);
+
+	/* Add a video file and set all information */
+	brasero_video_disc_add_uri_real (BRASERO_VIDEO_DISC (obj),
+					 uri,
+					 index,
+					 -1,
+					 -1,
+					 NULL);
+}
+
+static void
+brasero_video_disc_add_directory_contents (BraseroVideoDisc *self,
+					   const gchar *uri,
+					   BraseroTrack *sibling)
+{
+	BraseroVideoDiscPrivate *priv;
+	BraseroSessionCfg *session;
+	GtkTreeModel *model;
+	GSList *tracks;
+
+	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
+
+	if (!priv->load_dir)
+		priv->load_dir = brasero_io_register (G_OBJECT (self),
+						      brasero_video_disc_add_directory_contents_result,
+						      NULL,
+						      NULL);
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
+	session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (model));
+	tracks = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (session));
+
+	brasero_io_load_directory (uri,
+				   priv->load_dir,
+				   BRASERO_IO_INFO_MIME|
+				   BRASERO_IO_INFO_PERM|
+				   BRASERO_IO_INFO_METADATA|
+				   BRASERO_IO_INFO_METADATA_MISSING_CODEC|
+				   BRASERO_IO_INFO_RECURSIVE|
+				   BRASERO_IO_INFO_METADATA_THUMBNAIL,
+				   GINT_TO_POINTER (g_slist_index (tracks, sibling)));
+}
+
+static gboolean
+brasero_video_disc_directory_dialog (BraseroVideoDisc *self)
+{
+	gint answer;
+	GtkWidget *dialog;
+
+	dialog = brasero_app_dialog (brasero_app_get_default (),
+				     _("Do you want to search for video files inside the directory?"),
+				     GTK_BUTTONS_NONE,
+				     GTK_MESSAGE_WARNING);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("Directories cannot be added to video discs."));
+
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				_("_Search Directory"), GTK_RESPONSE_OK,
+				NULL);
+
+	gtk_widget_show_all (dialog);
+	answer = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	if (answer != GTK_RESPONSE_OK)
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+brasero_video_disc_unreadable_uri_dialog (BraseroVideoDisc *self,
+					  const gchar *uri,
+					  GError *error)
+{
+	gchar *primary;
+	gchar *name;
+
+	name = g_filename_display_basename (uri);
+	primary = g_strdup_printf (_("\"%s\" could not be opened."), name);
+	brasero_app_alert (brasero_app_get_default (),
+			   primary,
+			   error->message,
+			   GTK_MESSAGE_ERROR);
+	g_free (primary);
+	g_free (name);
+}
+
+static void
+brasero_video_disc_not_video_dialog (BraseroVideoDisc *self,
+				     const gchar *uri)
+{
+	gchar *primary;
+	gchar *name;
+
+	BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
+	primary = g_strdup_printf (_("\"%s\" does not have a suitable type for video projects."), name);
+	brasero_app_alert (brasero_app_get_default (),
+			   primary,
+			   _("Please only add files with video contents"),
+			   GTK_MESSAGE_ERROR);
+	g_free (primary);
+	g_free (name);
+}
+
+static void
+brasero_video_disc_session_changed (BraseroSessionCfg *session,
+				    BraseroVideoDisc *self)
+{
+	GSList *next;
+	GSList *tracks;
+	gboolean notready;
+	BraseroStatus *status;
+	BraseroVideoDiscPrivate *priv;
+
+	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
+
+	if (!GTK_WIDGET (self)->window)
+		return;
+
+	/* make sure all tracks have video */
+	notready = FALSE;
+	status = brasero_status_new ();
+	tracks = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (session));
+	for (; tracks; tracks = next) {
+		BraseroStreamFormat format;
+		BraseroTrackStream *track;
+		BraseroBurnResult result;
+
+		track = tracks->data;
+		next = tracks->next;
+
+		if (!BRASERO_IS_TRACK_STREAM (track))
+			continue;
+
+		result = brasero_track_get_status (BRASERO_TRACK (track), status);
+		if (result == BRASERO_BURN_ERR) {
+			GError *error;
+			gboolean res;
+			gchar *uri;
+
+			uri = brasero_track_stream_get_source (track, TRUE);
+			error = brasero_status_get_error (status);
+			if (!error)
+				brasero_video_disc_unreadable_uri_dialog (self, uri, error);
+			else if (error->code != BRASERO_BURN_ERROR_FILE_FOLDER) {
+				res = brasero_video_disc_directory_dialog (self);
+				if (res)
+					brasero_video_disc_add_directory_contents (self, uri, BRASERO_TRACK (track));
+			}
+			else if (error->code == BRASERO_BURN_ERROR_FILE_NOT_FOUND) {
+				/* It could be a file that was deleted */
+				brasero_video_disc_unreadable_uri_dialog (self, uri, error);
+			}
+			else
+				brasero_video_disc_unreadable_uri_dialog (self, uri, error);
+
+			brasero_burn_session_remove_track (BRASERO_BURN_SESSION (session),
+							   BRASERO_TRACK (track));
+			g_error_free (error);
+			g_free (uri);
+			continue;
+		}
+
+		if (result == BRASERO_BURN_NOT_READY) {
+			notready = TRUE;
+			continue;
+		}
+
+		if (result != BRASERO_BURN_OK)
+			continue;
+
+		format = brasero_track_stream_get_format (track);
+		if (!BRASERO_STREAM_FORMAT_HAS_VIDEO (format)) {
+			gchar *uri;
+
+			uri = brasero_track_stream_get_source (track, TRUE);
+			brasero_video_disc_not_video_dialog (self, uri);
+			g_free (uri);
+
+			brasero_burn_session_remove_track (BRASERO_BURN_SESSION (session),
+							   BRASERO_TRACK (track));
+		}
+	}
+	brasero_status_free (status);
+}
+
 static void
 brasero_video_disc_delete_selected (BraseroDisc *self)
 {
 	BraseroVideoDiscPrivate *priv;
 	GtkTreeSelection *selection;
+	BraseroSessionCfg *session;
 	GtkTreeModel *model;
 	GList *selected;
 	GList *iter;
@@ -386,19 +511,22 @@ brasero_video_disc_delete_selected (BraseroDisc *self)
 
 	selected = gtk_tree_selection_get_selected_rows (selection, &model);
 	selected = g_list_reverse (selected);
+
+	session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (model));
+
 	for (iter = selected; iter; iter = iter->next) {
-		BraseroVideoFile *file;
+		BraseroTrack *track;
 		GtkTreePath *treepath;
 
 		treepath = iter->data;
 
-		file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+		track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
 		gtk_tree_path_free (treepath);
 
-		if (!file)
+		if (!track)
 			continue;
 
-		brasero_video_project_remove_file (BRASERO_VIDEO_PROJECT (model), file);
+		brasero_burn_session_remove_track (BRASERO_BURN_SESSION (session), track);
 	}
 	g_list_free (selected);
 }
@@ -420,13 +548,13 @@ brasero_video_disc_get_selected_uri (BraseroDisc *self,
 		return FALSE;
 
 	if (uri) {
-		BraseroVideoFile *file;
+		BraseroTrack *track;
 		GtkTreePath *treepath;
 
 		treepath = selected->data;
-		file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
-		if (file)
-			*uri = g_strdup (file->uri);
+		track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+		if (track)
+			*uri = brasero_track_stream_get_source (BRASERO_TRACK_STREAM (track), TRUE);
 		else
 			*uri = NULL;
 	}
@@ -451,12 +579,14 @@ brasero_video_disc_selection_function (GtkTreeSelection *selection,
 				       gboolean path_currently_selected,
 				       gpointer NULL_data)
 {
-	BraseroVideoFile *file;
+	BraseroTrack *track;
 
-	file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
-	if (file)
+	track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+
+	/* FIXME: add a tag?? */
+/*	if (track)
 		file->editable = !path_currently_selected;
-
+*/
 	return TRUE;
 }
 
@@ -472,16 +602,18 @@ brasero_video_disc_rename_songs (GtkTreeModel *model,
 				 const gchar *old_name,
 				 const gchar *new_name)
 {
-	BraseroVideoFile *file;
+	BraseroTrack *track;
 
-	file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
-	if (!file)
+	track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+	if (!track)
 		return FALSE;
 
-	if (file->info->title)
-		g_free (file->info->title);
+	brasero_track_tag_add_string (track,
+				      BRASERO_TRACK_STREAM_TITLE_TAG,
+				      new_name);
 
-	file->info->title = g_strdup (new_name);
+	/* Signal the change to have the view reflect it */
+	brasero_track_changed (track);
 	return TRUE;
 }
 
@@ -534,32 +666,29 @@ brasero_video_disc_edit_song_properties_list (BraseroVideoDisc *self,
 						 &isrc,
 						 NULL);
 
-	/* start by the end in case we add silences since then the next
-	 * treepaths will be wrong */
 	copy = g_list_copy (list);
 	copy = g_list_reverse (copy);
 
 	for (item = copy; item; item = item->next) {
 		GtkTreePath *treepath;
-		BraseroVideoFile *file;
+		BraseroTrack *track;
 
 		treepath = item->data;
-		file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
-		if (!file)
+		track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+		if (!track)
 			continue;
 
-		if (artist) {
-			g_free (file->info->artist);
-			file->info->artist = g_strdup (artist);
-		}
+		brasero_track_tag_add_string (track,
+					      BRASERO_TRACK_STREAM_ARTIST_TAG,
+					      artist);
 
-		if (composer) {
-			g_free (file->info->composer);
-			file->info->composer = g_strdup (composer);
-		}
+		brasero_track_tag_add_string (track,
+					      BRASERO_TRACK_STREAM_COMPOSER_TAG,
+					      composer);
 
-		if (isrc > 0)
-			file->info->isrc = isrc;
+		brasero_track_tag_add_int (track,
+					   BRASERO_TRACK_STREAM_ISRC_TAG,
+					   isrc);
 	}
 
 	g_list_free (copy);
@@ -572,7 +701,7 @@ end:
 
 static void
 brasero_video_disc_edit_song_properties_file (BraseroVideoDisc *self,
-					      BraseroVideoFile *file)
+					      BraseroTrack *track)
 {
 	gint isrc;
 	gint64 end;
@@ -581,6 +710,7 @@ brasero_video_disc_edit_song_properties_file (BraseroVideoDisc *self,
 	gchar *artist;
 	gchar *composer;
 	GtkWidget *props;
+	guint64 length = 0;
 	GtkWidget *toplevel;
 	GtkTreeModel *model;
 	GtkResponseType result;
@@ -591,16 +721,18 @@ brasero_video_disc_edit_song_properties_file (BraseroVideoDisc *self,
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
 
+	brasero_track_stream_get_length (BRASERO_TRACK_STREAM (track), &length);
+
 	props = brasero_song_props_new ();
 	brasero_song_props_set_properties (BRASERO_SONG_PROPS (props),
 					   -1,
-					   file->info->artist,
-					   file->info->title,
-					   file->info->composer,
-					   file->info->isrc,
-					   file->end - file->start,
-					   file->start,
-					   file->end,
+					   brasero_track_tag_lookup_string (track, BRASERO_TRACK_STREAM_ARTIST_TAG),
+					   brasero_track_tag_lookup_string (track, BRASERO_TRACK_STREAM_TITLE_TAG),
+					   brasero_track_tag_lookup_string (track, BRASERO_TRACK_STREAM_COMPOSER_TAG),
+					   brasero_track_tag_lookup_int (track, BRASERO_TRACK_STREAM_ISRC_TAG),
+					   length,
+					   brasero_track_stream_get_start (BRASERO_TRACK_STREAM (track)),
+					   brasero_track_stream_get_end (BRASERO_TRACK_STREAM (track)),
 					   -1);
 
 	gtk_window_set_transient_for (GTK_WINDOW (props),
@@ -625,35 +757,37 @@ brasero_video_disc_edit_song_properties_file (BraseroVideoDisc *self,
 					   NULL);
 
 	if (title) {
-		if (file->info->title)
-			g_free (file->info->title);
-
-		file->info->title = title;
-		file->title_set = TRUE;
+		brasero_track_tag_add_string (track,
+					      BRASERO_TRACK_STREAM_TITLE_TAG,
+					      title);
+		g_free (title);
 	}
 
 	if (artist) {
-		if (file->info->artist)
-			g_free (file->info->artist);
-
-		file->info->artist = artist;
-		file->artist_set = TRUE;
+		brasero_track_tag_add_string (track,
+					      BRASERO_TRACK_STREAM_ARTIST_TAG,
+					      artist);
+		g_free (artist);
 	}
 
 	if (composer) {
-		if (file->info->composer)
-			g_free (file->info->composer);
-
-		file->info->composer = composer;
-		file->composer_set = TRUE;
+		brasero_track_tag_add_string (track,
+					      BRASERO_TRACK_STREAM_COMPOSER_TAG,
+					      composer);
+		g_free (composer);
 	}
 
-	if (isrc) {
-		file->info->isrc = isrc;
-		file->isrc_set = TRUE;
-	}
+	if (isrc)
+		brasero_track_tag_add_int (track,
+					   BRASERO_TRACK_STREAM_ISRC_TAG,
+					   isrc);
 
-	brasero_video_project_resize_file (BRASERO_VIDEO_PROJECT (model), file, start, end);
+	brasero_track_stream_set_boundaries (BRASERO_TRACK_STREAM (track),
+					     start,
+					     end,
+					     0);
+	/* Signal the change */
+	brasero_track_changed (track);
 
 end:
 
@@ -678,14 +812,14 @@ brasero_video_disc_edit_information_cb (GtkAction *action,
 		return;
 
 	if (g_list_length (list) == 1) {
-		BraseroVideoFile *file;
+		BraseroTrack *track;
 		GtkTreePath *treepath;
 
 		treepath = list->data;
 
-		file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
-		if (file)
-			brasero_video_disc_edit_song_properties_file (self, file);
+		track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+		if (track)
+			brasero_video_disc_edit_song_properties_file (self, track);
 	}
 	else
 		brasero_video_disc_edit_song_properties_list (self, list);
@@ -710,17 +844,19 @@ brasero_video_disc_open_file (BraseroVideoDisc *self)
 	list = gtk_tree_selection_get_selected_rows (selection, &model);
 
 	for (item = list; item; item = item->next) {
-		BraseroVideoFile *file;
+		BraseroTrack *track;
+		gchar *uri;
 
 		treepath = item->data;
-		file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+		track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
 		gtk_tree_path_free (treepath);
 
-		if (!file)
+		if (!track)
 			continue;
 
-		if (file->uri)
-			uris = g_slist_prepend (uris, file->uri);
+		uri = brasero_track_stream_get_source (BRASERO_TRACK_STREAM (track), TRUE);
+		if (uri)
+			uris = g_slist_prepend (uris, uri);
 	}
 	g_list_free (list);
 
@@ -743,7 +879,10 @@ brasero_video_disc_clipboard_text_cb (GtkClipboard *clipboard,
 	gchar **array;
 	gchar **item;
 
-	array = g_strsplit_set (text, "\n\r", 0);
+	if (!text)
+		return;
+
+	array = g_uri_list_extract_uris (text);
 	item = array;
 	while (*item) {
 		if (**item != '\0') {
@@ -765,6 +904,7 @@ brasero_video_disc_clipboard_text_cb (GtkClipboard *clipboard,
 
 		item++;
 	}
+	g_strfreev (array);
 }
 
 static void
@@ -773,26 +913,10 @@ brasero_video_disc_clipboard_targets_cb (GtkClipboard *clipboard,
 					 gint n_atoms,
 					 BraseroVideoDisc *self)
 {
-	GdkAtom *iter;
-	gchar *target;
-
-	iter = atoms;
-	while (n_atoms) {
-		target = gdk_atom_name (*iter);
-
-		if (!strcmp (target, "x-special/gnome-copied-files")
-		||  !strcmp (target, "UTF8_STRING")) {
-			gtk_clipboard_request_text (clipboard,
-						    (GtkClipboardTextReceivedFunc) brasero_video_disc_clipboard_text_cb,
-						    self);
-			g_free (target);
-			return;
-		}
-
-		g_free (target);
-		iter++;
-		n_atoms--;
-	}
+	if (brasero_clipboard_selection_may_have_uri (atoms, n_atoms))
+		gtk_clipboard_request_text (clipboard,
+					    (GtkClipboardTextReceivedFunc) brasero_video_disc_clipboard_text_cb,
+					    self);
 }
 
 static void
@@ -897,13 +1021,13 @@ brasero_video_disc_button_pressed_cb (GtkTreeView *tree,
 
 		brasero_disc_selection_changed (BRASERO_DISC (self));
 		if (event->type == GDK_2BUTTON_PRESS) {
-			BraseroVideoFile *file;
+			BraseroTrack *track;
 			GtkTreeModel *model;
 
 			model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree));
-			file = brasero_video_tree_model_path_to_file (BRASERO_VIDEO_TREE_MODEL (model), treepath);
-			if (file)
-				brasero_video_disc_edit_song_properties_file (self, file);
+			track = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (model), treepath);
+			if (track)
+				brasero_video_disc_edit_song_properties_file (self, track);
 		}
 	}
 
@@ -950,7 +1074,6 @@ brasero_video_disc_add_ui (BraseroDisc *disc,
 						      -1,
 						      &error);
 	if (!merge_id) {
-		BRASERO_BURN_LOG ("Adding ui elements failed: %s", error->message);
 		g_error_free (error);
 		return 0;
 	}
@@ -1015,57 +1138,6 @@ brasero_video_disc_key_released_cb (GtkTreeView *tree,
 	return FALSE;
 }
 
-static void
-brasero_video_disc_row_deleted_cb (GtkTreeModel *model,
-				   GtkTreePath *path,
-				   BraseroVideoDisc *self)
-{
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	brasero_disc_contents_changed (BRASERO_DISC (self),
-				       brasero_video_project_get_file_num (BRASERO_VIDEO_PROJECT (model)));
-}
-
-static void
-brasero_video_disc_row_inserted_cb (GtkTreeModel *model,
-				    GtkTreePath *path,
-				    GtkTreeIter *iter,
-				    BraseroVideoDisc *self)
-{
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	brasero_disc_contents_changed (BRASERO_DISC (self),
-				       brasero_video_project_get_file_num (BRASERO_VIDEO_PROJECT (model)));
-}
-
-static void
-brasero_video_disc_row_changed_cb (GtkTreeModel *model,
-				   GtkTreePath *path,
-				   GtkTreeIter *iter,
-				   BraseroVideoDisc *self)
-{
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	brasero_disc_contents_changed (BRASERO_DISC (self),
-				       brasero_video_project_get_file_num (BRASERO_VIDEO_PROJECT (model)));
-}
-
-static void
-brasero_video_disc_size_changed_cb (BraseroVideoProject *project,
-				    BraseroVideoDisc *self)
-{
-	brasero_disc_size_changed (BRASERO_DISC (self), brasero_video_project_get_size (project));
-}
-
 static void
 brasero_video_disc_init (BraseroVideoDisc *object)
 {
@@ -1073,62 +1145,17 @@ brasero_video_disc_init (BraseroVideoDisc *object)
 	GtkTreeSelection *selection;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
-	GtkTreeModel *model;
 	GtkWidget *mainbox;
 	GtkWidget *scroll;
 
 	priv = BRASERO_VIDEO_DISC_PRIVATE (object);
 
-	/* the information displayed about how to use this tree */
-	priv->notebook = brasero_disc_get_use_info_notebook ();
-	gtk_widget_show (priv->notebook);
-	gtk_box_pack_start (GTK_BOX (object), priv->notebook, TRUE, TRUE, 0);
-
-	mainbox = gtk_vbox_new (FALSE, 12);
+	mainbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (mainbox);
-	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), mainbox, NULL);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
+	gtk_box_pack_start (GTK_BOX (object), mainbox, TRUE, TRUE, 0);
 
-	/* Tree */
-	model = GTK_TREE_MODEL (brasero_video_tree_model_new ());
-
-	g_signal_connect (G_OBJECT (model),
-			  "row-deleted",
-			  G_CALLBACK (brasero_video_disc_row_deleted_cb),
-			  object);
-	g_signal_connect (G_OBJECT (model),
-			  "row-inserted",
-			  G_CALLBACK (brasero_video_disc_row_inserted_cb),
-			  object);
-	g_signal_connect (G_OBJECT (model),
-			  "row-changed",
-			  G_CALLBACK (brasero_video_disc_row_changed_cb),
-			  object);
-
-	g_signal_connect (G_OBJECT (model),
-			  "size-changed",
-			  G_CALLBACK (brasero_video_disc_size_changed_cb),
-			  object);
-	g_signal_connect (G_OBJECT (model),
-			  "not-video-uri",
-			  G_CALLBACK (brasero_video_disc_not_video_dialog),
-			  object);
-	g_signal_connect (G_OBJECT (model),
-			  "directory-uri",
-			  G_CALLBACK (brasero_video_disc_directory_dialog),
-			  object);
-	g_signal_connect (G_OBJECT (model),
-			  "unreadable-uri",
-			  G_CALLBACK (brasero_video_disc_unreadable_uri_dialog),
-			  object);
-	g_signal_connect (G_OBJECT (model),
-			  "vfs-activity",
-			  G_CALLBACK (brasero_video_disc_vfs_activity_changed),
-			  object);
-
-	priv->tree = gtk_tree_view_new_with_model (model);
+	priv->tree = gtk_tree_view_new ();
 	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (priv->tree));
-	g_object_unref (G_OBJECT (model));
 	gtk_widget_show (priv->tree);
 
 	g_signal_connect (priv->tree,
@@ -1151,7 +1178,7 @@ brasero_video_disc_init (BraseroVideoDisc *object)
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_add_attribute (column, renderer,
-					    "pixbuf", BRASERO_VIDEO_TREE_MODEL_MIME_ICON);
+					    "pixbuf", BRASERO_VIDEO_TREE_MODEL_THUMBNAIL);
 
 	renderer = gtk_cell_renderer_text_new ();
 	g_signal_connect (G_OBJECT (renderer), "edited",
@@ -1233,40 +1260,18 @@ brasero_video_disc_init (BraseroVideoDisc *object)
 						GDK_ACTION_MOVE);
 }
 
-static void
-brasero_video_disc_reset_real (BraseroVideoDisc *self)
-{
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	brasero_video_project_reset (project);
-	brasero_video_disc_vfs_activity_changed (project, FALSE, self);
-}
-
-static void
-brasero_video_disc_clear (BraseroDisc *disc)
-{
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (disc);
-
-	brasero_video_disc_reset_real (BRASERO_VIDEO_DISC (disc));
-
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
-	brasero_disc_size_changed (disc, 0);
-}
-
-static void
-brasero_video_disc_reset (BraseroDisc *disc)
-{
-	brasero_video_disc_reset_real (BRASERO_VIDEO_DISC (disc));
-}
-
 static void
 brasero_video_disc_finalize (GObject *object)
 {
+	BraseroVideoDiscPrivate *priv;
+
+	priv = BRASERO_VIDEO_DISC_PRIVATE (object);
+
+	if (priv->load_dir) {
+		g_free (priv->load_dir);
+		priv->load_dir = NULL;
+	}	
+
 	G_OBJECT_CLASS (brasero_video_disc_parent_class)->finalize (object);
 }
 
@@ -1311,124 +1316,38 @@ brasero_video_disc_set_property (GObject * object,
 }
 
 static BraseroDiscResult
-brasero_video_disc_get_status (BraseroDisc *self,
-			       gint *remaining,
-			       gchar **current_task)
-{
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	return brasero_video_project_get_status (project, remaining, current_task);
-}
-
-BraseroDiscResult
 brasero_video_disc_set_session_contents (BraseroDisc *self,
 					 BraseroBurnSession *session)
 {
-	GSList *tracks, *iter;
-	BraseroVideoProject *project;
+	BraseroVideoTreeModel *model;
 	BraseroVideoDiscPrivate *priv;
 
 	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	tracks = brasero_video_project_get_contents (project, FALSE);
 
-	if (!tracks)
-		return BRASERO_DISC_ERROR_EMPTY_SELECTION;
+	if (!session) {
+		GtkTreeModel *model;
+		BraseroSessionCfg *session;
 
-	for (iter = tracks; iter; iter = iter->next) {
-		BraseroTrack *track;
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
+		session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (model));
+		g_signal_handlers_disconnect_by_func (session,
+						      brasero_video_disc_session_changed,
+						      self);
 
-		track = iter->data;
-		brasero_burn_session_add_track (session, track);
-
-		/* It's good practice to unref the track afterwards as we don't
-		 * need it anymore. BraseroBurnSession refs it. */
-		g_object_unref (track);
-
-	}
-	g_slist_free (tracks);
-
-	return BRASERO_DISC_OK;
-}
-
-static BraseroDiscResult
-brasero_video_disc_get_track (BraseroDisc *disc,
-			      BraseroDiscTrack *disc_track)
-{
-	GSList *iter;
-	GSList *tracks;
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	disc_track->type = BRASERO_PROJECT_TYPE_VIDEO;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (disc);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	tracks = brasero_video_project_get_contents (project, TRUE);
-
-	for (iter = tracks; iter; iter = iter->next) {
-		BraseroDiscSong *song;
-		BraseroTrack *track;
-
-		track = iter->data;
-
-		song = g_new0 (BraseroDiscSong, 1);
-		song->uri = brasero_track_stream_get_source (BRASERO_TRACK_STREAM (track), TRUE);
-		song->start = brasero_track_stream_get_start (BRASERO_TRACK_STREAM (track));
-		song->end = brasero_track_stream_get_end (BRASERO_TRACK_STREAM (track));
-
-		song->info = g_new0 (BraseroStreamInfo, 1);
-		song->info->artist = g_strdup (brasero_track_tag_lookup_string (BRASERO_TRACK (track),
-										BRASERO_TRACK_STREAM_ARTIST_TAG));
-		song->info->title = g_strdup (brasero_track_tag_lookup_string (BRASERO_TRACK (track),
-									       BRASERO_TRACK_STREAM_TITLE_TAG));
-		song->info->composer = g_strdup (brasero_track_tag_lookup_string (BRASERO_TRACK (track),
-										  BRASERO_TRACK_STREAM_COMPOSER_TAG));
-		song->info->isrc = brasero_track_tag_lookup_int (BRASERO_TRACK (track),
-								 BRASERO_TRACK_STREAM_ISRC_TAG);
-
-		disc_track->contents.tracks = g_slist_append (disc_track->contents.tracks, song);
+		gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree), NULL);
+		return BRASERO_DISC_OK;
 	}
 
-	g_slist_foreach (tracks, (GFunc) g_object_unref, NULL);
-	g_slist_free (tracks);
+	model = brasero_video_tree_model_new ();
+	brasero_video_tree_model_set_session (model, BRASERO_SESSION_CFG (session));
+	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree),
+				 GTK_TREE_MODEL (model));
+	g_object_unref (model);
 
-	return BRASERO_DISC_OK;
-}
-
-static BraseroDiscResult
-brasero_video_disc_load_track (BraseroDisc *disc,
-			       BraseroDiscTrack *track)
-{
-	GSList *iter;
-	BraseroVideoProject *project;
-	BraseroVideoDiscPrivate *priv;
-
-	g_return_val_if_fail (track->type == BRASERO_PROJECT_TYPE_VIDEO, FALSE);
-
-	if (track->contents.tracks == NULL)
-		return BRASERO_DISC_ERROR_EMPTY_SELECTION;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (disc);
-	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
-	priv->loading = g_slist_length (track->contents.tracks);
-
-	for (iter = track->contents.tracks; iter; iter = iter->next) {
-		BraseroDiscSong *song;
-
-		song = iter->data;
-
-		brasero_video_project_add_uri (BRASERO_VIDEO_PROJECT (project),
-					       song->uri,
-					       song->info,
-					       NULL,
-					       song->start,
-					       song->end);
-	}
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 1);
+	g_signal_connect (session,
+			  "is-valid",
+			  G_CALLBACK (brasero_video_disc_session_changed),
+			  self);
 
 	return BRASERO_DISC_OK;
 }
@@ -1438,15 +1357,9 @@ brasero_video_disc_iface_disc_init (BraseroDiscIface *iface)
 {
 	iface->add_uri = brasero_video_disc_add_uri;
 	iface->delete_selected = brasero_video_disc_delete_selected;
-	iface->clear = brasero_video_disc_clear;
-	iface->reset = brasero_video_disc_reset;
 
-	iface->get_status = brasero_video_disc_get_status;
 	iface->set_session_contents = brasero_video_disc_set_session_contents;
 
-	iface->get_track = brasero_video_disc_get_track;
-	iface->load_track = brasero_video_disc_load_track;
-
 	iface->get_selected_uri = brasero_video_disc_get_selected_uri;
 	iface->add_ui = brasero_video_disc_add_ui;
 }
